#ifndef ZRE_VM_H
#define ZRE_VM_H

#include "common.h"

typedef union {
  float f;
  uint32_t u;
  char c[4];
} Data;

typedef enum {
  OP_HALT,           /* terminate execution */
  OP_ADD,            /* dest = src1 + src2  */
  OP_SUB,            /* dest = src1 - src2  */
  OP_MUL,            /* dest = src1 * src2  */
  OP_DIV,            /* dest = src1 / src2  */
  OP_JEQ_INT,        /* jump to address dest if src1 as int == src2 as int */
  OP_JGT_INT,        /* jump to address dest if src1 as int > src2 as int*/
  OP_JLT_INT,        /* jump to address dest if src1 as int < src2 as int */
  OP_JLE_INT,        /* jump to address dest if src1 as int <= src2 as int */
  OP_JGE_INT,        /* jump to address dest if src1 as int >= src2 as int*/
  OP_INT_TO_REAL,    /* dest = src1 as f32  */
  OP_ADD_REAL,       /* dest = src1 + src2  */
  OP_SUB_REAL,       /* dest = src1 - src2  */
  OP_MUL_REAL,       /* dest = src1 * src2  */
  OP_DIV_REAL,       /* dest = src1 / src2  */
  OP_JEQ_REAL,       /* jump to address dest if src1 as real == src2 as real */
  OP_JGE_REAL,       /* jump to address dest if src1 as real >= src2 as real */
  OP_JGT_REAL,       /* jump to address dest if src1 as real > src2 as real */
  OP_JLT_REAL,       /* jump to address dest if src1 as real < src2 as real */
  OP_JLE_REAL,       /* jump to address dest if src1 as real <= src2 as real */
  OP_REAL_TO_INT,    /* dest = src1 as int  */
  OP_MOV,            /* dest = src1	   */
  OP_JMP,            /* jump to address src1 unconditionally */
  OP_INT_TO_STRING,  /* dest = src1 as str */
  OP_REAL_TO_STRING, /* dest = src2 as str */
  OP_READ_STRING,    /* dest = src1 */
  OP_PRINT_STRING,   /* dest = src1 */
  OP_CMP_STRING,     /* dest = src1 */
} Opcode;

uint32_t step_vm(Data *memory, uint32_t memory_size, uint32_t pc);

#endif
